// File: SocketServer.java
// TCP/IP threaded Java server example that recieves an event request remotely
//	and creates an event to be fired locally

import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.event.*;
import javax.swing.*;

import java.io.*;
import java.net.*;

class SocketThrdServer15000 extends JFrame implements ActionListener {

   JLabel label = new JLabel("Text received over socket:");
   JPanel panel,ButtonPanel;
   JButton BList[];
   JTextArea textArea; 
   ServerSocket server = null;
   int port;

  // A ClientWorker is started each time a client requests service
  //	Each client has a ClientWorker that services its requests 
  class ClientWorker implements Runnable {
     private Socket client;
     private JTextArea textArea;
     
     ClientWorker(Socket client, JTextArea textArea) {
      this.client = client;
      this.textArea = textArea;   
     }

     public void run(){
       String line;
       BufferedReader in = null;
       PrintWriter out = null;
       // Set up streams for the socket
       try{
         in=new BufferedReader(new InputStreamReader(client.getInputStream()));
         out=new PrintWriter(client.getOutputStream(), true);
       } 
       catch (IOException e) {
         System.out.println("in or out failed");
         System.exit(-1);
       }
       // Communications
       while(true){
        try{
            line = in.readLine();		// Incoming Data
            int number=Integer.parseInt(line);	// contains a number
	    //Send data back to client
            System.out.println("Read: "+number);// To console
            // Report event to come in textarea on server GUI
            textArea.append("Pushing Button Number "+number+"\n");
            // Declare an action event; must construct something  
	    ActionEvent e=new ActionEvent(BList[0],0,"");
	    switch (number) {
	     // Now, construct the event to be sent
	     case 1: 
	      e=new ActionEvent(BList[0],ActionEvent.ACTION_PERFORMED,"First");
	      break;
	     case 2: 
	      e=new ActionEvent(BList[1],ActionEvent.ACTION_PERFORMED,"Second");
	    }
	    // Call actionPerformed()
	    actionPerformed(e);
        } 
	catch    (IOException e) {
            System.out.println("Read failed");
            System.exit(-1);
        }
       }
     }
  }

   SocketThrdServer15000(){ //Begin Constructor
     // getContentPane().setLayout(null);
     getContentPane().setBounds(20,20,250,500);
     setBounds(20,20,250,500);
     // To report events
     textArea = new JTextArea();
     textArea.setSize(250,200);
     // Allow textarea to expand beyond bounds
     JScrollPane TextScroll=new JScrollPane(textArea);
     panel = new JPanel();
     panel.setSize(250,300);
     panel.setLayout(new BorderLayout());
     panel.setBackground(Color.white);
     getContentPane().add(panel);
     panel.add("North", label);
     panel.add("Center", TextScroll);
     // Put the buttons to be pushed remotely onto a panel
     ButtonPanel=new JPanel();
     ButtonPanel.setLayout(new BorderLayout());
     ButtonPanel.setBackground(Color.blue);
     BList=new JButton[2];
     BList[0]=new JButton("First");
     BList[0].addActionListener(this);
     ButtonPanel.add("West",BList[0]);
     BList[1]=new JButton("Second");
     BList[1].addActionListener(this);
     ButtonPanel.add("East",BList[1]);
     panel.add("South",ButtonPanel);
     
   } //End Constructor

   public void actionPerformed(ActionEvent e)
   { JButton Source=(JButton)e.getSource();
     Color Old=Source.getBackground();
     // Turn button green for visual effect
     Source.setBackground(Color.green);
     System.out.println("Button "+e.getActionCommand()+ " pushed: ");
     // sleep 0.25 second to make visual effect visible 
     try {
        Thread.sleep(250);
     } catch (Exception exc) {}
     // Restore button color
     Source.setBackground(Old);
   }

   public void listenSocket(int port)
   {// First, get a socket; use the port passed in
    try{
      server = new ServerSocket(port); 
    } 
    catch (IOException e) {
      System.out.println("Could not listen on port "+port);
      System.exit(-1);
    }
    // Handle service requests
    while(true){
      ClientWorker w;
      try{
	// accept() fires when a request comes in; blocks until then
        w = new ClientWorker(server.accept(), textArea);
	// Set up a thread to handle this client's requests
        Thread t = new Thread(w);
	// execute the thread's run() method;
        t.start();    
	// then, back to wait for another client's request
      } 
      catch (IOException e) {
        System.out.println("Accept failed: Port #"+port);
        System.exit(-1);
      }
    }
  }

  protected void finalize(){
  //Objects created in run method are finalized when 
  //program terminates and thread exits
     try{
        server.close();
    } 
    catch (IOException e) {
        System.out.println("Could not close socket");
        System.exit(-1);
    }
  }

  public static void main(String[] args){
        SocketThrdServer15000 frame = new SocketThrdServer15000();
	frame.setTitle("Server Program");
        WindowListener l = new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                        System.exit(0);
                }
        };
        frame.addWindowListener(l);
	frame.setSize(200,200);
//        frame.pack();
        frame.setVisible(true);

	// Frame is ready; let it begin being a server
        frame.listenSocket(15000);
  }
}

